const chalk = require('chalk')
const theapps = require('../utils/apps.js')
const cliSelect = require('cli-select')
const log = require('../utils/log.js')
const prompt = require('prompt')
const getPrompt = require('util').promisify(prompt.get).bind(prompt)

module.exports = async (args) => {
  log('Removing Aptugo application', { type: 'mainTitle' })

  const apps = await theapps.list()
  if (!args.app && !args._[1]) {
    log('\nSelect the application to remove:', { type: 'promptHeader' })
    const appSelected = await cliSelect({
      values: apps,
      indentation: 2,
      cleanup: true,
      selected: '▹',
      unselected: '',
      valueRenderer: (value, selected) => {
        if (selected) {
          return chalk.underline(value.settings.name)
        } else {
          return `${value.settings.name}`
        }
      }
    })
    args.app = appSelected.value
  } else if (args._[1]) {
    args.app = args._[1]
  }


  if (typeof args.app === 'string') {
    args.app = apps.find(app => app._id === args.app)
  }

  // Confirmation
  if (!args.confirmed) {
    prompt.start()
    appname = await getPrompt({
      description: `Are you sure you wish to delete, erase, remove ${args.app.settings.name} (YES/NO)`,
      name: 'confirm',
      importd: true,
      validator: /YES|NO/,
      warning: 'You must type YES or CTRL+C buddy',
    })

    if (appname.confirm === 'YES') {
      return theapps.remove(args.app)
    } else {
      log('Your application survived', { type: 'error', exit: true })
      return 'ok'
    }
  } else {
    return theapps.remove(args.app)
  }
}
