const log = require('../utils/log.js')
const theapps = require('../utils/apps.js')
const scraper = require('../utils/scraper.js')
const cliSelect = require('cli-select')
const chalk = require('chalk')

const plainPages = []
const getPlainPages = (tree) => {
  tree.forEach(item => {
    if (item.type === 'page') {
      Object.keys(item).map(propertyName => {
        if (item[propertyName] && item[propertyName].substr && item[propertyName].substr(0,2) === '()') {
          let replacedValue = item[propertyName].replace('aptugo.store.getState().application.tables','params.plainTables')
          replacedValue = replacedValue.replace('aptugo.activeApplication.tables','params.plainTables')
          replacedValue = '(params)' + replacedValue.substr(2)
          item[propertyName] = parseToString(replacedValue)
        }
      })
      plainPages.push({ ...item, children: [] })
    }
    if (item.children && item.children.length) item.children = getPlainPages(item.children)
  })

  return plainPages
}

const getPlainStylesheets = (assets) => {
  return assets.filter(asset => asset.type === 'stylesheet')
}

const selectApp = async (args) => {
  if (!args.app && !args._[1]) {
    log('\nSelect the application to load:', { type: 'promptHeader', verbosity: 0  })
    const apps = await theapps.list()
    const appSelected = await cliSelect({
      values: apps,
      indentation: 2,
      cleanup: true,
      selected: '▹',
      unselected: '',
      valueRenderer: (value, selected) => {
        if (selected) {
          return chalk.underline(value.settings.name)
        } else {
          return `${value.settings.name}`
        }
      }
    })
    args.app = appSelected.value
  } else if (args._[1]) {
    args.app = args._[1]
  }
  return args
}

const selectPage = async (app, args) => {
  if (!args.page) {
    log('\nSelect the page to import to:', { type: 'promptHeader', verbosity: 0  })
    const pages = getPlainPages(JSON.parse(JSON.stringify(app.pages)))
    const pageSelected = await cliSelect({
      values: pages,
      indentation: 2,
      cleanup: true,
      selected: '▹',
      unselected: '',
      valueRenderer: (value, selected) => {
        if (selected) {
          return chalk.underline(value.name)
        } else {
          return `${value.name}`
        }
      }
    })
    return aptugocli.plain[pageSelected.value.unique_id]
  }
  return aptugocli.plain[args.page]
}

const selectAsset = async (app, args) => {
  if (!args.asset) {
    log('\nSelect the stylesheet to import to:', { type: 'promptHeader', verbosity: 0  })
    const stylesheets = getPlainStylesheets(JSON.parse(JSON.stringify(app.assets)))
    const stylesheetSelected = await cliSelect({
      values: stylesheets,
      indentation: 2,
      cleanup: true,
      selected: '▹',
      unselected: '',
      valueRenderer: (value, selected) => {
        if (selected) {
          return chalk.underline(value.name)
        } else {
          return `${value.name}`
        }
      }
    })
    return stylesheetSelected.value
  } else {
    return app.assets.filter(asset => asset.id === args.asset)[0]
  }
}

module.exports = async (args, extra) => {
  log('Converting HTML page to Aptugo', { type: 'mainTitle', verbosity: 0 })

  args = await selectApp(args)
  args.app = theapps.load(args.app)
  args.page = await selectPage(args.app, args)
  args.asset = await selectAsset(args.app, args)

  return scraper.scrape(args)
}
