const fs = require('fs')
const path = require('path')
const templates = require('../templates.js')
const fsLoadAndParseFile = templates.fsLoadAndParseFile
const twig = require('twig/twig.js')
const log = require('../log.js')
const error = require('../error.js')
const twigRender = require('./twigRender.js')
const { isBinary } = require('istextorbinary')
const config = require('../config.js')

module.exports = (parameters) => {
  const doCopyStaticFiles = (parameters) => {
    return new Promise((resolve,reject) => {
      parameters.files.forEach(prefile => {
        let skip = false
        if (prefile.path === 'elements' || prefile.path === 'Fields' || prefile.path === 'templatescripts') skip = true

        if (!skip) {
          let fileName = prefile.path
          let partialPath = path.join(parameters.accumulated || '', fileName )
          let sourcePath = path.join( config.get('folders').templates ,aptugocli.activeParameters.template._id, partialPath )
          let fullPathWithoutFile = prefile.fullPathWithoutFile || path.join(parameters.fullbuildfolder, parameters.buildFolder, parameters.accumulated || '')
          let fullPath = prefile.fullPath || path.join(parameters.fullbuildfolder, parameters.buildFolder, partialPath)


          if ( isBinary(sourcePath) ) {
            log(`Copying binary file: ${fullPath}`, { type: 'advance', level: parameters.level, verbosity: 7 })
            fs.copyFileSync( sourcePath, fullPath )
          } else {
            let [postfile,fs] = fsLoadAndParseFile({ unique_id: prefile.unique_id })
            const file = { ...prefile, ...postfile }
            aptugocli.currentFile = {
              ...file,
              fullPath: fullPath,
              fullPathWithoutFile: fullPathWithoutFile
            }

            if (file.modelRelated) {
              for (var table in parameters.application.tables) {
                if (file.subtype === 'Any' || (file.subtype === parameters.application.tables[table].subtype)) {
                  let calculatedFilename = twig.twig({ data: path.join(file.path) })
                  let modelfileName = calculatedFilename.render({ table: parameters.application.tables[table] })
                  let modelPartialPath = path.join(parameters.accumulated || '', modelfileName )
                  fullPath = prefile.fullPathWithoutFile ? path.join(prefile.fullPathWithoutFile, modelfileName): path.join(parameters.fullbuildfolder, parameters.buildFolder, modelPartialPath)
                  aptugocli.currentFile = {
                    ...file,
                    fullPath: fullPath,
                    fullPathWithoutFile: fullPathWithoutFile
                  }
                  parameters.table = parameters.application.tables[table]
                  parameters.table.definedRoutes?.map(definedRoute => {
                    if (definedRoute.route_code) {
                      definedRoute.route_code = twigRender({ data: definedRoute.route_code }, parameters)
                    }
                    return definedRoute
                  })
                  
                  if (file.type === 'folder') {
                    aptugocli.createIfDoesntExists(fullPath)
                    if (file.children && file.children.length) {
                      doCopyStaticFiles({
                        ...parameters,
                        level: parameters.level + 1,
                        files: [...file.children]
                      })
                    }
                  } else {
                    log(`Copying model file: ${fullPath}`, { type: 'advance', level: parameters.level, verbosity: 7 })
                    try {
                      var contents = twigRender({ data: fs || ''}, parameters)
                      aptugocli.writeFile( fullPath, contents, true )
                    } catch(e) {
                      console.error('Error compiling template for file: ', fileName, e)
                    }
                  }
                }
              }
            } else if (file.type === 'folder') {
              aptugocli.createIfDoesntExists(fullPath)
              log(`Creating folder: ${fullPath}`, { type: 'advance', level: parameters.level, verbosity: 7 })
              if (file.children && file.children.length) {
                doCopyStaticFiles({
                  ...parameters,
                  files: [...file.children],
                  accumulated: partialPath,
                  level: parameters.level + 1
                })
              }
            } else {
              try {
                var contents = twigRender({ data: fs || ''}, parameters)
                log(`Copying static file: ${fullPath}`, { type: 'advance', level: parameters.level, verbosity: 7 })
                aptugocli.writeFile( fullPath, contents, true )
              } catch(e) {
                error('Error compiling template for file: ', fileName, e)
              }
            }
          }
        }
      })
      resolve()
    })
  }
  return doCopyStaticFiles({ ...parameters, level: 0, files: parameters.files || parameters.template.files }).then(res => {
    return 'ok'
  })
}
