const fs = require('fs')
const path = require('path')
const yaml = require('js-yaml')
const { isBinary } = require('istextorbinary')

const fsParseFileForStorage = (fileDetails) => {
  const {
    source,
    open,
    selected,
    selectedFile,
    ...cleanFileDetails
  } = fileDetails

  let output = '/*\n'
  output += yaml.dump(cleanFileDetails)
  output += '*/\n'
  output += fileDetails.source
  return output
}

exports.getTree = (folder, accumulatedPath = '', fix = false) => {
  const ignoreFiles = ['template.json','.DS_Store','_.yaml','templatescripts']
  const output = []
  const filesInFolder = fs.readdirSync(folder)
  filesInFolder.forEach(file => {
    if (ignoreFiles.indexOf(file) === -1) {
      let toPush = {
        path: file,
        completePath: path.join(folder, file),
        keyPath: path.join(accumulatedPath, file)
      }
      if (fs.lstatSync( path.join(folder, file) ).isDirectory()) {
        toPush.type = 'folder'
        toPush.unique_id = aptugocli.generateID()
        toPush.children = module.exports.getTree( path.join(folder, file), path.join(accumulatedPath, file), fix )
        try {
          const folderSource = fs.readFileSync( path.join(folder, file, '_.yaml'), 'utf8')
          toPush = { ...toPush, ...yaml.load(folderSource) }
        } catch(e) {}
      } else {
        let binary = isBinary(path.join(folder, file))
        if (file.substring(file.length - 4) === '.svg') binary = true
        if (!binary) {
          const fileSource = fs.readFileSync( path.join(folder, file), 'utf8')
          const [prefs, source] = module.exports.parseFile(fileSource)
          prefs.completePath = path.join(folder, file)
          toPush = { ...toPush, ...prefs }
          if (prefs.fullload) toPush.source = source
          if (fix) {
            if (!prefs.unique_id) { // Add and save unique_id if not exists
              toPush.unique_id = aptugocli.generateID()
              fs.writeFileSync(path.join(folder, file), fsParseFileForStorage({ ...toPush, source }), { flag: 'w' } )
            }
          } else {
            toPush.unique_id = prefs.unique_id || aptugocli.generateID()
          }
        }
      }
      output.push(toPush)
    }
  })
  return output
}

exports.mergeTree = (original, key = 'keyPath') => {
  const output = []
  original.forEach((value, index, completeArray) => {
    var foundMatching = completeArray.findIndex(value2 => value2[key] === value[key])
    if (foundMatching > -1 && foundMatching !== index) { // Found duplicate
    } else {
      if (value.type === 'folder') { // Check Folder
        var sliced = completeArray.slice(index + 1)
        var foundMatchingLast = sliced.findIndex(value2 => value2[key] === value[key])
        if (foundMatchingLast > -1) {
          value.children = exports.mergeTree([ ...value.children, ...sliced[foundMatchingLast].children], key)
        }
      }
      output.push(value)
    }
  })
  return output
}

exports.write = ({ saveFolder, filename, content, clean = false }) => {
  const cleanForSaving = (input, folder) => {
    let output = []
    input.forEach(actualObject => {
      const { unique_id, ...rest } = actualObject
      const { children, ...actualObjectWithoutChildren } = actualObject

      // Sort Properties
      const ordered = Object.keys(actualObjectWithoutChildren).sort().reduce(
        (obj, key) => {
          obj[key] = actualObjectWithoutChildren[key]
          return obj
        },
        {}
      )

      // Remove Known Properties
      const removableProperties = ['completePath','errored','helpText','icon','keyPath','options']
      removableProperties.forEach(propToRemove => {
        delete ordered[propToRemove]
      })

      fs.writeFileSync( path.join( saveFolder, folder, `${unique_id}.json`), JSON.stringify(ordered, null, 2), { flag: 'w' } )

      const localOutput = { unique_id }
      if (actualObject.children && actualObject.children.length > 0) localOutput.children = cleanForSaving(actualObject.children, folder)
      output.push(localOutput)
    })
    return output
  }

  if (clean) content = cleanForSaving(content, clean)
  fs.writeFileSync(
    path.join(saveFolder, filename),
    JSON.stringify(content, null, 2),
    { flag: 'w' }
  )
}

exports.parseFile = (fileSource) => {
  const regex = /^\/\*[\r]*\n(.*)[\r]*\n\*\/(.*)/is
  let m
  if ((m = regex.exec(fileSource)) !== null) {
    var yamlStuff = ''
    try {
      yamlStuff = yaml.load(m[1])
    } catch(e) {
      console.error(fileSource, e)
    }
    return [ yamlStuff , m[2]]
  }
  return [{}, fileSource]
}
