// const puppeteer = require('puppeteer')
const os = require('os')
const path = require('path')
const h2a = require('html-to-ast')
const theapps = require('./apps.js')
const theassets = require('./assets')
const prettier = require('prettier')
const parserScss = require('prettier/parser-postcss.js')
const orderScss = require('prettier-plugin-css-order')
const { structures } = require('./structures')
const request = require('request')
const fs = require('fs')
const csstree = require('css-tree')
const log = require('./log.js')

async function autoScroll(page){
  await page.evaluate(async () => {
      await new Promise((resolve, reject) => {
          var totalHeight = 0;
          var distance = 100;
          var timer = setInterval(() => {
              var scrollHeight = document.body.scrollHeight;
              window.scrollBy(0, distance);
              totalHeight += distance;

              if(totalHeight >= scrollHeight - window.innerHeight){
                  clearInterval(timer);
                  resolve();
              }
          }, 100);
      });
  });
}

const navigateAndChange = (need, asset, tree) => {
  tree.forEach(leaf => {
    if (leaf.value === 'image' && path.basename(leaf.values.path) === need) {
      leaf.values.useAsset = asset
    }
    if (leaf.children) {
      navigateAndChange(need, asset, leaf.children)
    }
  })
}

const download = function(uri, filename) {
  return new Promise((resolve,reject) => {
    request.head(uri, function(err, res, body) {
      console.log('content-type:', res.headers['content-type']);
      console.log('content-length:', res.headers['content-length']);

      const ws = fs.createWriteStream(filename)
      request(uri).pipe(ws).on('close', () => {
        resolve()
      })
    })
  })
}

function startBrowser(headless) {
  return new Promise(async (resolve, reject) => {
    let browser;
    try {
      console.info("Opening the browser......");
      // browser = await puppeteer.launch({
      //   dumpio: false,
      //   headless: false,
      //   args: ["--disable-setuid-sandbox",'--no-sandbox'],
      //   'ignoreHTTPSErrors': true
      // })
    } catch (err) {
      console.info("Could not create a browser instance => : ", err)
      reject(err)
    }
    resolve(browser)
  })
}

const scraperObject = {
  url: undefined,
  entirePage: false,
  images: [],
  stylesheets: [],
	async scraper(browser, callback){
		let page = await browser.newPage()
		console.log(`Navigating to ${this.url}...`)
    await page.setViewport({
      width: parseInt(960),
      height: parseInt(1080),
      deviceScaleFactor: 0.5,
    })
    await page.setRequestInterception(true)

    page.on('request', request => {
      if (request.url().indexOf('googlesyndication') > -1 || request.url().indexOf('px.ads.linkedin.com') > -1 ) {
        request.abort()
      } else {
        request.continue()
      }
    })

    await page.exposeFunction('__callback', callback)
    page.on('response', async response => {
      const url = response.url()
      const responseType = response.request().resourceType()
      let ignore = response.request().url().indexOf('googlesyndication') > -1 ? true : false
      ignore = response.request().url().indexOf('collect') > -1 ? true : ignore
      ignore = response.request().url().indexOf('bat.bing.com') > -1 ? true : ignore
      ignore = response.request().url().indexOf('connect.facebook.net') > -1 ? true : ignore
      ignore = response.request().url().indexOf('www.facebook.com/tr') > -1 ? true : ignore
      ignore = response.request().url().indexOf('analytics.twitter.com') > -1 ? true : ignore
      ignore = response.request().url().indexOf('t.co/i/adsct') > -1 ? true : ignore
      ignore = response.request().url().indexOf('trackId') > -1 ? true : ignore
      ignore = response.request().url().length > 300 ? true : ignore

      if (!ignore) {
        switch (responseType) {
          case 'document':
            break
          case 'font':
            break
          case 'image':
            const fileName = path.basename(url)
            const destination = path.join(os.tmpdir(),'AptugoScrapper',fileName)
            this.images.push(fileName)
            response.buffer().then(file => {
              try {
                const writeStream = fs.createWriteStream(destination)
                writeStream.write(file)
                console.log('Wrote Image:', destination)
              } catch(e) {
                console.log('Error with image:', destination)
              }
            })
            break
          case 'stylesheet':
            const stylesheetName = path.basename(url)
            const stylesheetDestination = path.join(os.tmpdir(),'AptugoScrapper',stylesheetName)
            this.stylesheets.push(stylesheetDestination)
            response.buffer().then(file => {
              try {
                const writeStream = fs.createWriteStream(stylesheetDestination)
                writeStream.write(file)
                console.log('Wrote Image from Stylesheet:', stylesheetDestination)
              } catch(e) {
                console.log('Error with stylesheet:', stylesheetDestination)
              }
            })
            break
          case 'xhr':
          case 'ping':
            break
          case 'script':
            break
          default:
            console.log('Do not know how to handle response', response.request().resourceType(), ' for url:', url)
        }
      }

      // console.log('response', url, response.request().resourceType())
    })


    await page.goto(this.url, { waitUntil: 'networkidle0' })
    await autoScroll(page)

    for (var stylesheet in this.stylesheets) {
      console.log('STYLE:', this.stylesheets[stylesheet])
      await page.addStyleTag({ path: this.stylesheets[stylesheet] })
    }

    await page.evaluate((args) => {
      function GetAppliedCssRules($element) {
        var appliedRules = [];
        if (!document.styleSheets) {
          console.warn('Can not read stylesheets')
          return appliedRules
        }
        for (var x = 0; x < document.styleSheets.length; x++) {
          var rules = null
          try {
            rules = document.styleSheets[x].cssRules
          } catch (e) {
            console.warn("Can't read the css rules of: " + document.styleSheets[x].href)
            continue
          }

          if (rules) {
            for (var i = 0; i < rules.length; i++) {
              if (rules[i].media) {
                appliedRules.push(rules[i].cssText)
              }
              if ($element.matches(rules[i].selectorText)) {
                appliedRules.push(rules[i].cssText)
              }
            }
          }
        }
        return appliedRules;
      }

      if (args.entirePage) {
        // Removes External Stylesheets
        try {
          for (var x = 0; x < document.styleSheets.length; x++) {
            if ( document.styleSheets[x].href ) {
              // const failedStylesheet = document.styleSheets[x].href.substring(document.styleSheets[x].href.lastIndexOf('/') + 1)
              // const replacementStylesheet = args.stylesheets.find(ss => ss.indexOf(failedStylesheet))
              document.styleSheets[x].ownerNode.parentNode.removeChild(document.styleSheets[x].ownerNode)
              // document.styleSheets[x].href = replacementStylesheet
            }
          }
        } catch(e) {
          console.error('Failed to remove external stylesheets: ', e)
        }

        var element = document.getElementsByTagName('body')[0]
        var els = element.getElementsByTagName("*")

        // Get all used CSS rules and Images for every element in the page
        const usedRules = []
        const allImages = []
        for(var i = -1, l = els.length; ++i < l;) {
          var currentElement = els[i]
          if (currentElement.tagName === 'IMG') {
            allImages.push({ src: currentElement.src, fullSrc: currentElement.currentSrc })
          }
          if (currentElement.tagName !== 'STYLE') {
            console.log('current element', currentElement)
            try {
              usedRules.push( GetAppliedCssRules(currentElement) )
            } catch(e) {
              console.warn('heres jhonny', e)
            }

          }
        }
        __callback(element.outerHTML, usedRules, allImages)
      } else { // Allows to select the element to import
        var overelem = null
        // Add overlay
        var style = document.createElement('style')
        style.innerHTML = '.aptugooverlay { background-color: rgba(208,110,29,.24); border: 1px solid orange; position: absolute; top: 0; left: 0; width: 100px; height: 100px; z-index: 12003023; }'
        document.head.appendChild(style)

        function getOffset(el) {
          const rect = el.getBoundingClientRect();
          return {
            left: rect.left + window.scrollX,
            top: rect.top + window.scrollY,
            width: rect.width,
            height: rect.height
          };
        }

        function addOverLay(el) {
          var overlay = document.querySelector('.aptugooverlay')
          if (!overlay) {
            overlay = document.createElement('div')
            overlay.classList.add('aptugooverlay')
            document.body.appendChild(overlay)

            overlay.addEventListener('click', (e) => {
              var element = overelem
              var els = element.getElementsByTagName("*")

              const usedRules = []
              const allImages = []
              for(var i = -1, l = els.length; ++i < l;) {
                var currentElement = els[i]
                if (currentElement.tagName === 'IMG') {
                  allImages.push({ src: currentElement.src, fullSrc: currentElement.currentSrc })
                }
                if (currentElement.tagName !== 'STYLE') {
                  usedRules.push( GetAppliedCssRules(currentElement) )
                }
              }
              __callback(overelem.outerHTML, usedRules, allImages)
            })
          }
          overlay.style.left = (getOffset(el).left - 1) + 'px'
          overlay.style.top = (getOffset(el).top - 1) + 'px'
          overlay.style.width = (getOffset(el).width + 2) + 'px'
          overlay.style.height = (getOffset(el).height + 2) + 'px'
        }

        let hover = document.querySelectorAll('div,nav,section,p')
        for (let elem of hover) {
          elem.addEventListener('mouseover', (e) => {
            overelem = elem // document.getElementsByTagName('body')[0]
            addOverLay(elem)
            e.stopPropagation()
            e.preventDefault()
          })
        }
      }
    },this)
	}
}

const navigateTree = (tree, parent) => {
  let output = []
  tree.forEach(branch => {
    let doPush = false
    let toPush = {
      unique_id: aptugocli.generateID(),
      parent: parent,
      path: 'div.tpl',
      name: 'imported',
      type: 'element',
      value: 'div',
      values: {},
      children: []
    }

    // console.log(branch)
    if (branch.type === 'tag') {
      if (branch.name === 'a') {
        toPush.path = 'simpleLink.tpl'
        toPush.value = 'simpleLink'
        toPush.values = { destination: "'" + branch.attrs.href + "'"}
        doPush = true
      } else if (branch.name === 'svg' || branch.name === 'path' || branch.name === 'clipboard-copy' || branch.name === 'style') {
        doPush = false
      } else if (branch.name === 'script') {
        toPush.path = 'script.tpl'
        toPush.value = 'script'
        doPush = false
      } else if (branch.name === 'img') {
        toPush.path = 'image.tpl'
        toPush.value = 'image'
        toPush.values = { path: branch.attrs.src, alt: branch.attrs.alt }
        doPush = true
      } else if (branch.name === 'h6' || branch.name === 'h2') {
        toPush.path = 'typography.tpl'
        toPush.value = 'typography'
        toPush.values = { tag: branch.name }
        toPush.children = [{ type: 'element', unique_id: aptugocli.generateID(), value: 'text', parent: toPush.unique_id, name: branch.innerHTML, values: { Content: branch.innerHTML } }]
        doPush = true
      } else {
        toPush.values = { tag: branch.name }
        toPush.name = 'imported ' + branch.name
        doPush = true
      }

      if (branch.attrs?.class) {
        const classes = branch.attrs?.class.split(/\s+/).filter(a => a !== '')
        var camelCased = classes.join('_').replace(/[-:]([a-z0-9])/g, function (g) { return g[1].toUpperCase(); })
        if (classes.length === 1) {
          toPush.values.class = "theme['" + classes[0] + "']"
        } else {
          toPush.values.class = '`${theme["' + classes.join('"]} ${theme["') + '"]}`'
        }
        toPush.values.className = toPush.values.class
        toPush.name = camelCased
      }
    } else if (branch.type === 'text' && branch.content.match(/\S+/g)) {
      const cleanText = branch.content.replace(/^\s+|\s+$|\s+(?=\s)/g, "")
      if (cleanText) {
        toPush.path = 'text.tpl'
        toPush.value = 'text'
        toPush.values = { Content: cleanText }
        doPush = true
      }
    }

    if (branch.children) toPush.children = navigateTree(branch.children, toPush.unique_id)
    doPush && output.push(toPush)
  })
  return output
}

module.exports = {
  scrape: (args) => {
    return new Promise(async (resolve, reject) => {
      console.info(`Starting to scrape: ${args.url}`)
      startBrowser(args.entirePage ? true : false).then(browser => {
        scraperObject.url = args.url
        scraperObject.entirePage = args.entirePage ? true : false
        scraperObject.scraper(browser, async (receivedHTML, cssRules, allImages) => {
          await browser.close()

          // Build Tree
          console.log('Building DOM Tree')
          const astParsed = h2a.parse(receivedHTML)
          const aptugoTree = navigateTree(astParsed, args.page.children[2].unique_id)
          if (!args.page.children[2].children) args.page.children[2].children = []
          args.page.children[2].children = [...args.page.children[2].children, ...aptugoTree]

          // Build Styles
          const reducedRules = []
          cssRules.forEach(rule => {
            rule.forEach(subRule => {
              reducedRules.push(subRule)
            })
          })
          console.log(`Building ${reducedRules.length} Styles`)

          let cleanRules = [...new Set(reducedRules)]
          cleanRules.sort()
          cleanRules = cleanRules.join('\n')

          let newStyleContents = '/* Imported Styles */\n\n'
          newStyleContents += args.replaceCss ? '' : args.asset.fileContents
          newStyleContents += cleanRules
          const ast = csstree.parse(newStyleContents)

          csstree.walk(ast, {
            enter(node) {
              if (node.type === 'Url') {
                node.value = '/img' + node.value.substring(node.value.lastIndexOf('/'))
                return node
              }
            }
          })
          newStyleContents = csstree.generate(ast)

          newStyleContents = prettier.format(newStyleContents, {
            semi: false,
            parser: 'scss',
            endOfLine: 'lf',
            importOrder: ["^@core/(.*)$", "^@server/(.*)$", "^@ui/(.*)$", "^[./]"],
            importOrderSeparation: true,
            plugins: [parserScss, orderScss]
          })



        // Import Images
        const importImages = async () => {
          for (var I = 0; I < scraperObject.images.length; I++) {
            console.log('Importing Image', I, ' out of ', scraperObject.images.length)
            try {
              const filename = scraperObject.images[I]
              const destination = path.join(os.tmpdir(),'AptugoScrapper',filename)
              console.log('creating asset', filename, destination)
              const result = await structures.run('Create Asset', { type: 'image', name: filename, path: destination, state: { app: args.app } } )
              console.log('id of asset:', result.assets.at(-1).id )
              navigateAndChange(filename, result.assets.at(-1).id, args.app.pages)
            } catch(e) {
              console.log(e)
            }
          }
        }

        await importImages()
        console.log('finished')

        if (!args.dryrun) {
          if (!args.onlyCss) theapps.save(args.app)
          if (!args.onlyApp) await theassets.setfile({ binary: true, app: args.app._id, id: args.asset.id }, newStyleContents )
        } else {
          console.log('> Resulting Tree:', JSON.stringify(args.app.pages, null, 2))
          console.log('> Resulting Styles:', newStyleContents)
        }

        // args.app.pages = aptugocli.updatePageInTree(args.app.pages,args.page.unique_id, { ...args.page, children: [...args.page.children[2].children, ...aptugoTree] })
        resolve('ok')
        })
      }).catch(e => {
        reject(e)
      })
    })
    // return new Promise(async (resolve,reject) => {
    //   const browser = await startBrowser(args.entirePage ? true : false)
    //   scraperObject.url = args.url
    //   scraperObject.entirePage = args.entirePage ? true : false
    //   scraperObject.scraper(browser, async (receivedHTML, cssRules, allImages) => {
    //     const data = receivedHTML
    //     const astParsed = h2a.parse(data)
    //     await browser.close()

    //     // Build Tree
    //     console.log('Building Tree')
    //     const aptugoTree = navigateTree(astParsed, args.page.children[2].unique_id)
    //     args.page.children[2].children = [...args.page.children[2].children, ...aptugoTree]


    //   })
    // })
  }
}
