const fs = require('fs')
const os = require('os')
const path = require('path')
const yaml = require('js-yaml')
const config = require('./config.js')
const { getTree, mergeTree } = require('./files.js')
const axios = require('axios')
const { exit } = require('process')
const { isBinary } = require('istextorbinary')
const AdmZip = require('adm-zip')
const log = require('./log.js')
const request = require('request')
const { mergeDeepWith } = require('immutable');


function downloadBase (source, destination) {
  return new Promise((resolve, reject) => {
    var len = 0
    var cur = 0
    var total = 0
    var chunkBody
    log(`Starting templates download`, { type: 'advance', level: 0, verbosity: 1 })

    const destinationPath = path.join( os.tmpdir(), 'AptugoBase' )
    if (!fs.existsSync(destinationPath)) {
      fs.mkdirSync( destinationPath, { recursive: true, mode: 0o777 })
    } else { // Destination exists, move it!
      aptugocli.deleteFolderRecursive( path.join( os.tmpdir(), 'AptugoTemplatesBackup' ) )
      fs.renameSync( destinationPath,  path.join( os.tmpdir(), 'AptugoTemplatesBackup' ))
      fs.mkdirSync( destinationPath, { recursive: true, mode: 0o777 })
    }

    request.get({
      url: source,
      encoding: null,
      headers: {
        "User-Agent": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.97 Safari/537.11",
        "Referer": "http://www.nseindia.com/products/content/all_daily_reports.htm",
        "Accept-Encoding": "identity",
        "encoding": "null",
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
        "Cookie": "cookie"
      }
    }).pipe(fs.createWriteStream(path.join(destinationPath, 'downloaded.zip')))
    .on('close', function () {
      try {
        var zip = new AdmZip(path.join(destinationPath, 'downloaded.zip'))
        zip.extractAllTo(destinationPath, true, true)
      } catch(e) {
        reject(e)
      }

      const weirdName = fs.readdirSync(destinationPath, { encoding: 'utf-8'})
      console.log('weird name and stuff', weirdName)

      const templatesFolder = config.get('folders').templates
      try {
        if (fs.existsSync(templatesFolder)) {
          fs.renameSync(templatesFolder, templatesFolder + '_old')
          aptugocli.deleteFolderRecursive(templatesFolder + '_old')
        }
      } catch(e) {
        console.log('Unable to delete actual templates folder', templatesFolder, e)
      }

      console.log('Putting templates in place')
      try {
        fs.renameSync( path.join(destinationPath, weirdName[0], 'templates'), templatesFolder )
      } catch(e) {
        console.log('Could not move templates to final directory', path.join(destinationPath, weirdName[0], 'templates'), templatesFolder, e)
      }

      const structuresFolder = config.get('folders').structures
      try {
        if (fs.existsSync(structuresFolder)) {
          fs.renameSync(structuresFolder, structuresFolder + '_old')
          aptugocli.deleteFolderRecursive(structuresFolder + '_old')
        }
      } catch(e) {
        console.log('Unable to delete actual structures folder', structuresFolder, e)
      }

      console.log('Putting structures in place')
      try {
        fs.renameSync( path.join(destinationPath, weirdName[0], 'structures'), structuresFolder )
      } catch(e) {
        console.log('Could not move structures to final directory', path.join(destinationPath, weirdName[0], 'structures'), structuresFolder, e)
      }

      resolve(weirdName)
    }).on('data', (chunk) => {
      chunkBody += chunk
      cur += chunk.length
      console.log('Received ' + cur + ' bytes')
      console.info('Received ' + cur + ' bytes')
    }).on('error', (error) => {
      reject(error)
    })
  })
}

const templatesModule = {
  list: (loadfull = false) => {
    try {
      const toReturn = []
      const folders = config.get('folders')
      if (!folders.templates) {
        throw({
          exitCode: 150,
          message: 'Aptugo Client is not configured'
        })
      }
      let templateFolders = []
      try {
        templateFolders = fs.readdirSync(folders.templates)
      } catch(e) {
        throw({
          exitCode: 151,
          message: 'Templates folder missing'
        })
      }

      templateFolders.forEach(templateFolder => {
        try {
          if (fs.lstatSync( path.join(folders.templates,templateFolder) ).isDirectory()) {
            if (fs.existsSync(path.join(folders.templates,templateFolder,'template.json'))) {
              const fileContents = fs.readFileSync(path.join(folders.templates,templateFolder,'template.json'), { encoding: 'utf8'}, true)
              if (fileContents) {
                let templateDefinition = JSON.parse( fileContents )
                if ( loadfull ) {
                  templateDefinition = templatesModule.get(templateDefinition._id, templateFolder)
                }
                toReturn.push(templateDefinition)
              }
            }
          }
        } catch(e) {
          throw({
            exitCode: 171,
            error: e,
            message: 'Failed to read template folder ' + templateFolder
          })
        }
      })
      return toReturn
    } catch(e) {
      if (e.exitCode) throw(e)
      else {
        throw({
          exitCode: 170,
          error: e,
          message: 'Could not read templates folder (is it configured?)'
        })
      }
    }

  },
  version: (args) => {
    const folders = config.get('folders')
    if ( folders.templates && fs.existsSync( path.join(folders.templates, 'version.txt') ) ) {
      const version = fs.readFileSync( path.join(folders.templates, 'version.txt'), { encoding: 'utf8' } )
      return version
    } else {
      return 'unknown'
    }
  },
  create: (args) => {
    args = JSON.parse(args)
    const folders = config.get('folders')
    const templateDefinitionFolder = path.join(folders.templates, args._id)
    if (!fs.existsSync(templateDefinitionFolder)) {
      fs.mkdirSync(templateDefinitionFolder)
    }
    fs.writeFileSync(path.join(templateDefinitionFolder,'template.json'), JSON.stringify( args, null, 2 ), { flag:'w' })
    return args
  },
  remove: (args) => {
    const template = args.template
    const folders = config.get('folders')
    const tplFolders = folders.templates
    const saveFolder = path.join(tplFolders, template)
    fs.rmdirSync(saveFolder, { recursive: true })
    return 'ok'
  },
  setoption: (templateFolder, option, value) => {
    const folders = config.get('folders')
    const templateFilePath = path.join(folders.templates,templateFolder,'template.json')
    const templateContents = fs.readFileSync(templateFilePath)
    if (templateContents) {
      let templateDefinition = JSON.parse( templateContents )
      templateDefinition[option] = value
      fs.writeFileSync(templateFilePath, JSON.stringify(templateDefinition, null, 2), {flag:'w'})
    }
  },
  setfield: (templateFolder, fieldDefinition) => {
    const folders = config.get('folders')
    const templateFolderPath = path.join(folders.templates,templateFolder)
    const fieldsDefinitionFolder = path.join(templateFolderPath,'Fields')
    if (!fs.existsSync(fieldsDefinitionFolder)) {
      fs.mkdirSync(fieldsDefinitionFolder)
    }
    fs.writeFileSync(path.join(fieldsDefinitionFolder,`${fieldDefinition.name}.json`), JSON.stringify( JSON.parse(fieldDefinition.value), null, 2 ), { flag:'w' })
    return path.join(fieldsDefinitionFolder,`${fieldDefinition.name}.json`)
  },
  setfile: (templateFolder, fileDefinition) => {
    const fileDetails = JSON.parse(fileDefinition)
    const parsedFile = templatesModule.fsParseFileForStorage(fileDetails)
    const folders = config.get('folders')
    const templateFolderPath = path.join(folders.templates,templateFolder)

    const fileName = fileDetails.modelRelated ? fileDetails.completePath.replace(/{{(.*?)}}/g, `${fileDetails.subtype}[X]`) : fileDetails.completePath || fileDetails.path
    if (fileDetails.type === 'folder') {
      const folderPath = path.join(templateFolderPath, fileName)
      if (!fs.existsSync(folderPath)) {
        fs.mkdirSync(folderPath)
      }
      const { children, source, ...cleanFileDetails } = fileDetails
      fs.writeFileSync(path.join(folderPath,'_.yaml'), yaml.dump(cleanFileDetails))
    } else if (parsedFile) {
      fileDetails.source = parsedFile
      const filePath = path.join(templateFolderPath, fileName)
      fs.writeFileSync(filePath, fileDetails.source)
    }
  },
  renderPath: (file) => {
    if (file.modelRelated) {
      return file.path.replace(/{{(.*?)}}/g, `${file.subtype}[X]`)
    } else {
      return file.path
    }
  },
  fileSource: (templateID, fileID) => {
    const folders = config.get('folders')
    const templates = templatesModule.list()
    const currentTemplate = templates.filter(template => template._id === templateID)[0]
    currentTemplate.files = getTree( path.join( folders.templates, currentTemplate._id ), '', true )
    const file = templatesModule.findFileWithUniqueID(currentTemplate.files, fileID)
    if (!file) return 'Error: File not found'
    const fileSource = fs.readFileSync(path.join(folders.templates, currentTemplate._id, file.completePath,templatesModule.renderPath(file)), 'utf8')
    return fileSource
  },
  get: (templateID, templateFolder) => {
    const folders = config.get('folders')
    const thetemplates = templatesModule.list()
    const currentTemplate = thetemplates.filter(template => template._id === templateID)[0]
    currentTemplate.files = getTree( path.join( folders.templates, templateFolder || currentTemplate._id ), '', true )
    currentTemplate.fields = templatesModule.getFields(templateFolder || currentTemplate._id)


    // Check for extended templates
    if (folders.extendedTemplates) {
      try {
        if (fs.existsSync( path.join( folders.extendedTemplates, templateFolder || currentTemplate._id ) )) {
          const extendedFiles = getTree( path.join( folders.extendedTemplates, templateFolder || currentTemplate._id ), '', true )
          currentTemplate.files = mergeTree([ ...extendedFiles, ...currentTemplate.files])
        }
      } catch(e) {
        console.log('Error loading Extended Templates', e)
      }
    }
    return currentTemplate
  },
  getFields: (templateFolder) => {
    const output = []
    const folders = config.get('folders')
    const fieldsFolder = path.join( folders.templates, templateFolder, 'Fields' )
    if (fs.existsSync(fieldsFolder)) {
      const fieldsInFolder = fs.readdirSync( fieldsFolder )
      fieldsInFolder.forEach(fieldFileName => {
        if (fieldFileName.toLowerCase().substr(-4) === 'json') {
          const [discard, fieldSource] = templatesModule.fsParseFile( fs.readFileSync(path.join( fieldsFolder, fieldFileName ), 'utf8') )

          let parsed
          try {
            parsed = JSON.parse(fieldSource)
            output.push(parsed)
          } catch(e) {
            console.log('error loading ', fieldFileName, e, fieldSource)
          }
        }
      })
    }
    return output
  },
  fsParseFileForStorage(fileDetails) {
    const {
      source,
      open,
      selected,
      selectedFile,
      ...cleanFileDetails
    } = fileDetails

    let output = '/*\n'
    output += yaml.dump(cleanFileDetails)
    output += '*/\n'
    output += fileDetails.source
    return output
  },
  fsParseFile: (fileSource) => {
    const regex = /^\/\*[\r]*\n(.*)[\r]*\n\*\/(.*)/is
    let m
    if ((m = regex.exec(fileSource)) !== null) {
      var yamlStuff = ''
      try {
        yamlStuff = yaml.load(m[1])
      } catch(e) {
        console.error(fileSource, e)
      }
      return [ yamlStuff , m[2]]
    }
    return [{}, fileSource]
  },
  fsLoadAndParseFile: (params) => {
    const fileSource = templatesModule.fsLoadFileSource(params)
    return templatesModule.fsParseFile(fileSource)
  },
  fsLoadFileSource: (params) => {
    let template, file
    if (!params.file) {
      template = aptugocli.activeParameters.template
      file = params.unique_id
      ? templatesModule.findFileWithUniqueID(template.files, params.unique_id , path.join( config.get('folders').templates ,template._id))
      : templatesModule.findFileWithPath(template.files, params.path, path.join( config.get('folders').templates ,template._id))
    } else {
      file = params.file
    }
    if (!file) return ''

    const fileName = file.modelRelated ? file.completePath.replace(/{{(.*?)}}/g, `${file.subtype}[X]`) : file.completePath
    try {
      const fileSource = fs.readFileSync(fileName, 'utf8')
      file.source = fileSource
      return fileSource
    } catch(e) {
      return ''
    }
  },
  findFileWithUniqueID: (tree, unique_id, acumulated_path = '') => {
    if (tree) {
      for (var i = 0; i < tree.length; i++) {
        const filePath = tree[i].modelRelated ? tree[i].path.replace(/{{(.*?)}}/g, `${tree[i].subtype}[X]`) : tree[i].path
        if (tree[i].unique_id === unique_id) {
          return {
            ...tree[i]
          }
        }
        var found = templatesModule.findFileWithUniqueID(tree[i].children, unique_id, path.join(acumulated_path, filePath))
        if (found) return found
      }
    }
  },
  findFileWithPath: (tree, thepath, acumulated_path) => {
    if (tree) {
      for (var i = 0; i < tree.length; i++) {
        const filePath = tree[i].modelRelated ? tree[i].path.replace(/{{(.*?)}}/g, `${tree[i].subtype}[X]`) : tree[i].path
        if (tree[i].completePath === thepath) {
          return {
            ...tree[i],
            completePath: acumulated_path
          }
        }
        var found = templatesModule.findFileWithPath(tree[i].children, thepath, path.join(acumulated_path, filePath))
        if (found) return found
      }
    }
  },
  searchForRenderingPlaceholder: (tree, acumulated = '.') => {
    if (tree) {
      for (var i = 0; i < tree.length; i++) {
        if (tree[i].placeholder === true) {
          return acumulated + tree[i].path
        }
        var found = templatesModule.searchForRenderingPlaceholder(tree[i].children, acumulated + '/' + tree[i].path + '/')
        if (found) return found
      }
    }
  },
  // Returns available Templates version
  checkTemplatesVersion: (args) => {
    const remote = args.remote || 'https://api.github.com/repos/AptugosTeam/BaseDev/releases/latest'
    return axios.get(remote).then(res => {
      return res.data.tag_name
    }).catch(e => {
      return e
    })
  },
  download: (args) => {
    const remote = args.remote || 'https://api.github.com/repos/AptugosTeam/BaseDev/releases/latest'
    return axios.get(remote).then(res => {
      var downloadedVersion = module.exports.version()
      const releaseName = res.data.tag_name
      const sourceFile = res.data.zipball_url
      let destination = path.join( os.tmpdir(), 'aptugoBase.zip' )
      console.log('Templates actual version: ', downloadedVersion, ' - Available: ', releaseName)

      return downloadBase(sourceFile, destination).then(response => {
        const Messages = ['All your base are belong to us','That what I said','Everything should be ready now','And everrything for free!','Ready to rumble!','Things look good!']
        const selectedMessage = Messages[Math.floor(Math.random() * Math.floor(Messages.length))]
        if (typeof aptugo !== 'undefined') aptugo.setFeedback({ kind: 'alert', title: 'Downloaded Templates!', message: selectedMessage })
        return response
      }).catch(e => {
        return {
          exitCode: 188,
          message: 'Something nasty happened when trying to download templates',
          error: e
        }
      })

    }).catch(e => {
      return {
        exitCode: 187,
        message: 'Network Error when trying to download templates',
        error: e
      }
    })
  }
}

module.exports = templatesModule
